MUSIC_ADDRESS		EQU #4000
PLAYER_ADDRESS		EQU #8000
PROGRAM_ADDRESS		EQU #A000

include "toolbox.asm"

ORG PROGRAM_ADDRESS

	DISABLE_INT (INTER+1)   ; Desactive vecteur IT
	PUSH_MIRROR_REGS	    ; Sauve les registres miroirs

MAIN:
	LD DE,MUSIC_ADDRESS     ; Initialise musique
	CALL PLAYER_ADDRESS
MAINLOOP:
	WAITSYNC				; Attente trame vidéo	
	DI			
	GETKBSTATE KBSTATE		; Récupere état clavier
	EI	
	
	CALL PLAYER_ADDRESS + 3	; Joue le soundtrack	
	
	LD HL,(FRAMECNT)
	INC HL
	LD (FRAMECNT),HL	
	ld A,H
	ld (display),A		
	BIT 0,L					; Numero de Trame paire?
	JR Z,PAIR
	LD IX,VOLUMES2
	LD IY,VOLUMES1
	JR SETP
PAIR:
	LD IX,VOLUMES1
	LD IY,VOLUMES2
SETP:

	LD A,8			; Canal A
	CALL PSGREADREG
	AND #0F         ; VOLUME SUR 4 BITS
	LD (IX+0),A
	LD D,A			; On met de coté

	LD A,9			; Canal B
	CALL PSGREADREG
	AND #0F         ; VOLUME SUR 4 BITS
	LD (IX+1),A
    LD E,A 			; On met de coté 

    LD A,10			; Canal C
	CALL PSGREADREG
	AND #0F         ; VOLUME SUR 4 BITS
	LD (IX+2),A
	ADD D			; Somme Volumes A+B+C
	ADD E			
	SRA A			; Divisée par 4
	SRA A			; 
	LD (IX+3),A 	; Volume des 3 canaux cumulés

MACRO DRAWVOL CHAN,DEST
	LD HL,{DEST}
	LD A,(IX+{CHAN})
	CALL DRAWLINE
MEND

MACRO CALC_ATTACK CHAN
	LD A,(IX+{CHAN})
	LD D,(IY+{CHAN})
	SUB D
	DEC A
	JP P,@POS
	XOR A
@POS
	LD (IX+4+{CHAN}),A
MEND
	
	CALC_ATTACK 0
	CALC_ATTACK 1
	CALC_ATTACK 2
	CALC_ATTACK 3
	
;  INSERE LES DONNÉES
	LD HL,(SPECTRE_INDEX)
	push IX
	ld a,(display)
	bit 1,a
	jr nz, showatt
showvol:
	LD BC, PALETTE16
	jr insert
showatt:
	LD BC, PALETTE12
	
	inc IX 
	inc IX
	inc IX
	inc IX
insert:		
	N=0
REPEAT 3
	LD D,B				; DE=BC
	LD E,C
	LD A,(IX+N)			; A = donnée a injecter
	ADD E
	LD E,A
	LD A,(DE)			; Lecture dans la LUT (palette12)
	LD (HL),A
	INC H				; Canal suivant	LD D,B	
	N=N+1
REND
	pop IX

	HALT
	HALT
	;DEBUT DU BLOC RASTER
	DI				

	; LE  RASTER AFFECTE LA COULEUR 0
	LD      BC,GA_PORT | GA_SELECT_COL | 0
	OUT     (C),C

	LD C, 128
	LD HL,(SPECTRE_INDEX)
	DS 50

RASTV3_2:
	PUSH HL
	REPEAT 3
		LD E,(HL)
		INC H
		OUT (C),E												
	REND		
	NOP
	LD E,GA_BLUE
	OUT (C),E													
	POP HL
	DEC L             ; On passe a l'echantillon precedent
	DS 24
	DEC C
	JR NZ,RASTV3_2

	EI
	HALT
	
	LD DE,#0FF0
	; Choix de l'encre pour indiquer quel est celui selectionné
	LD A,(display)
	BIT 1,A	
	JR Z,selvol
	LD E,#FF
selvol:
	VUMOFFS EQU #C000+80*8
	DRAWVOL 0,VUMOFFS
	DRAWVOL 1,VUMOFFS+#1000
	DRAWVOL 2,VUMOFFS+#2000
	DRAWVOL 3,VUMOFFS+80
	
	LD A,E
	XOR #0F
	LD E,A
	
	ATTOFFS EQU #C000+80*11+320
	DRAWVOL 4,ATTOFFS
	DRAWVOL 5,ATTOFFS+#1000
	DRAWVOL 6,ATTOFFS+#2000
	DRAWVOL 7,ATTOFFS+80
		
;Deplacement de l'index pour la prochaine donnée
	LD HL,(SPECTRE_INDEX)
	INC L					; Simple incrementation de L
	LD (SPECTRE_INDEX),HL	; Le buffer faisant 256 octets
	
ENDLOOP:	
	LD 	A,(KBSTATE+5)       ; Touche ESPACE?
	BIT 7,A
	JP  NZ,MAINLOOP         ; Nouvelle boucle

EXIT:
	POP_MIRROR_REGS			; Rétablit les registres miroirs
INTER:  
	LD	HL,0				; Retablit les ITs d'origine 
	LD 	(#38),HL	
	CALL BIOS_RESET_AUDIO	; Silence
	RET						; Retour au basic

; Remplissage de la ligne
; CE: HL contient l'addresse du premier pixel
;     A contient la valeur 0-15 a representer
DRAWLINE:
	INC A
	LD B,A
	LD C,B
	LD A,#FF
@LP1:
	LD (HL),D
	INC HL
	DJNZ @LP1
	LD A,17
	SUB C
	LD B,A
	LD A,#0F
@LP2:
	LD (HL),E
	INC HL
	DJNZ @LP2
	RET

; Lecture d'un registre du processeur audio PSG AY3-8912
; CE: A contient le numéro de registre
; CS: A contient la valeur du registre, BC modifié
PSGREADREG:
	 LD BC,#F782
     OUT (C),C
     LD B,#F4     ; Sélectionne le
     OUT (C),A    ; Registre
     LD BC,#F6C0  
     OUT (C),C
     XOR A
     OUT (C),A
     LD BC,#F792
     OUT (C),C
     LD BC,#F640
     OUT (C),C
     LD B,#F4     ; Ici on vient lire
     IN A,(C)     ; sa valeur
     LD BC,#F782
     OUT (C),C
     LD BC,#F600
     OUT (C),C
	 RET
	
KBSTATE:   ds 10			; Etat du clavier
FRAMECNT:  dw 0				; Compteur de trame
VOLUMES1:  ds 8				; Stats sur le volume
VOLUMES2:  ds 8				; (double buffer)
SPECTRE_INDEX: DW SPECTRE
display: db 0

PALETTE16:
	DB GA_BLACK
	DB GA_BLUE
	DB GA_BLUE_BRIGHT
	DB GA_BLUE_SKY
	DB GA_WHITE
	DB GA_YELLOW_BRIGHT
	DB GA_WHITE
	DB GA_YELLOW_BRIGHT
	DB GA_ORANGE
	DB GA_RED
	DB GA_ORANGE
	DB GA_RED
	DB GA_WHITE
	DB GA_WHITE
	DB GA_WHITE
	
PALETTE12:
	DB GA_BLACK
	DB GA_GREY
	DB GA_GREY
	DB GA_BLUE_SKY
	DB GA_WHITE
	DB GA_WHITE	
	DB GA_YELLOW_BRIGHT
	DB GA_YELLOW
	DB GA_ORANGE
	DB GA_ORANGE
	DB GA_RED
	DB GA_RED
		

ORG #9000
SPECTRE: DS 256, GA_RED
         DS 256, GA_GREEN         
         DS 256, GA_BLUE_BRIGHT

 
ORG PLAYER_ADDRESS
	READ "arkos/arkostrackerplayer_cpc_msx.asm"
	;READ "arkos/arkostrackerplayer_cpcstable_msx.asm"

ORG MUSIC_ADDRESS
	INCBIN "SONG4000.bin"
